﻿DeclareModule Easygrid
  
  ;Grid Definition Structure
  Structure GridDef
    GridGadget.i
    Width.i
    Height.i
    AutoCol.i
    FirstVisibleRow.i
    LastVisibleRow.i
    FirstVisibleColumn.i
    LastVisibleColumn.i
    Array RowHeights.i(0)
    Array ColWidths.i(0)  
    Map RowPixels.i()
    Map ColPixels.i()
  EndStructure

  Global Dim GridsArray.GridDef(0)
  
  Declare New(Left.i,Top.i,Width.i,Height.i,Rows.i,Columns.i)
  Declare DrawGrid(Gadget.i,Array GridData(2))
  Declare.i SetAutoColresize(Grid.i,State.i)
  Declare.i GetRow(Gadget.i,Pos.i)
  Declare.i GetCol(Gadget.i,Pos.i) 
  Declare.i SetRowHeight(Grid.i,Row.i,Height.i )
  Declare.i SetColumnWidth(Grid.i,Column.i,Width.i )
  Declare.i SetFirstRow(Grid.i,Row.i)
  Declare.i SetFirstCol(Grid.i,Col.i)
  
EndDeclareModule

Module EasyGrid
  
  #defaultColumnWidth = 20
  
  Global DisplayGridHeight.i,DisplayGridWidth.i
  Global CellTop.i,CellLeft.i,Cellbottom.i,CellRight.i ;Hold Current Cell Pixel values
  
  Procedure.i SetAutoColresize(Grid.i,State.i)
    
    ThisGrid = -1
    For iLoop = 0 To   ArraySize(GridsArray()) - 1
      
      If Grid = GridsArray(iLoop)\GridGadget
        ThisGrid = iLoop
      EndIf
      
    Next iLoop
    If ThisGrid = -1 ;Grid not defined
      ProcedureReturn -1
    EndIf 
    
    GridsArray(ThisGrid)\AutoCol = State
    
    ProcedureReturn 1
    
  EndProcedure 
  
  Procedure.i SetFirstRow(Grid.i,Row.i)
    
    ThisGrid = -1
    For iLoop = 0 To   ArraySize(GridsArray()) - 1
      
      If Grid = GridsArray(iLoop)\GridGadget
        ThisGrid = iLoop
      EndIf
      
    Next iLoop
    If ThisGrid = -1 ;Grid not defined
      ProcedureReturn -1
    EndIf 
    
    GridsArray(ThisGrid)\FirstVisibleRow = Row
    
    ProcedureReturn 1
    
  EndProcedure 
  
  Procedure.i SetFirstCol(Grid.i,Col.i)
    
    ThisGrid = -1
    For iLoop = 0 To   ArraySize(GridsArray()) - 1
      
      If Grid = GridsArray(iLoop)\GridGadget
        ThisGrid = iLoop
      EndIf
      
    Next iLoop
    If ThisGrid = -1 ;Grid not defined
      ProcedureReturn -1
    EndIf 
    
    GridsArray(ThisGrid)\FirstVisibleColumn = Col
    
    ProcedureReturn 1
    
  EndProcedure 
  
  Procedure.i SetRowHeight(Grid.i,Row.i,Height.i )
    
    ThisGrid = -1
    For iLoop = 0 To   ArraySize(GridsArray()) - 1
      
      If Grid = GridsArray(iLoop)\GridGadget
        ThisGrid = iLoop
      EndIf
      
    Next iLoop
    If ThisGrid = -1 ;Grid not defined
      ProcedureReturn -1
    EndIf 
    
    GridsArray(ThisGrid)\RowHeights(Row) = Height
    
    ProcedureReturn 1
    
  EndProcedure
  
  Procedure.i SetColumnWidth(Grid.i,Column.i,Width.i )
    
    ThisGrid = -1
    For iLoop = 0 To   ArraySize(GridsArray()) - 1
      
      If Grid = GridsArray(iLoop)\GridGadget
        ThisGrid = iLoop
      EndIf
      
    Next iLoop
    If ThisGrid = -1 ;Grid not defined
      ProcedureReturn -1
    EndIf 
    
    GridsArray(ThisGrid)\ColWidths(Column) = Width
    
    ProcedureReturn 1
    
  EndProcedure
  
  Procedure GetGridDimensions(ThisGrid.i)   
   
  ;Calculate GridHeight
  DisplayGridHeight = GridsArray(ThisGrid)\RowHeights(0) ;Add Row 0
  For iLoop = GridsArray(ThisGrid)\FirstVisibleRow To ArraySize(GridsArray(ThisGrid)\RowHeights())
    DisplayGridHeight = DisplayGridHeight + GridsArray(ThisGrid)\RowHeights(iLoop) 
    If DisplayGridHeight => GridsArray(ThisGrid)\Height
      DisplayGridHeight = GridsArray(ThisGrid)\Height
      Break
    EndIf  
  Next iLoop 
  LastVisibleRow = iLoop - 1

  ;Calculate GridWidth
  DisplayGridWidth = GridsArray(ThisGrid)\ColWidths(0)
  For iLoop = GridsArray(ThisGrid)\FirstVisibleColumn To ArraySize(GridsArray(ThisGrid)\ColWidths())
    DisplayGridWidth = DisplayGridWidth + GridsArray(ThisGrid)\ColWidths(iLoop)
    If DisplayGridWidth > GridsArray(ThisGrid)\Width
      DisplayGridWidth = GridsArray(ThisGrid)\Width
      Break
    EndIf 
  Next iLoop 
  LastVisibleColumn = iLoop - 1

EndProcedure 
    
  Procedure.i GetRow(Gadget.i,Pos.i)
  ;{ ==GetRow Comment==============================
  ;App Global Procedure to return the cell row determined by the mouse 
  ;position on the grid canvas gadget.
  ;returns -1 on failure or the row number of the selected cell
  ;}   
    ThisGrid = -1
    For iLoop = 0 To   ArraySize(GridsArray()) - 1
      
      If Gadget = GridsArray(iLoop)\GridGadget
        ThisGrid = iLoop
      EndIf
      
    Next iLoop
    If ThisGrid = -1 ;gadget not defined as an EasyGrid So Fail
      ProcedureReturn -1
    EndIf
  
    If FindMapElement(GridsArray(ThisGrid)\RowPixels(), Str(Pos))
      ProcedureReturn GridsArray(ThisGrid)\RowPixels()    
    Else
      ProcedureReturn -1
    EndIf
  
  EndProcedure

  Procedure.i GetCol(Gadget.i,Pos.i)
    ;{ ==GetCol Comment==============================
    ;App Global Procedure to return the cell Column determined by the mouse 
    ;position on the grid canvas gadget
    ;}     
    ThisGrid = -1
    For iLoop = 0 To   ArraySize(GridsArray()) - 1
      
      If Gadget = GridsArray(iLoop)\GridGadget
        ThisGrid = iLoop
      EndIf
      
    Next iLoop
    If ThisGrid = -1 ;gadget not defined as an EasyGrid So Fail
      ProcedureReturn -1
    EndIf
    
    If FindMapElement(GridsArray(ThisGrid)\ColPixels(), Str(Pos))
      ProcedureReturn GridsArray(ThisGrid)\ColPixels()   
    Else
     ProcedureReturn -1
    EndIf
  
  EndProcedure

  Procedure SetDefaults(ThisGrid.i,Rows.i,Columns.i)
    ;{ ==SetDefaults Comment==============================
    ;Procedure to set default values for the active grid
    ;The procedure draws all grid lines and header highlights
    ;The LastVisibleRow and LastVisibleColumn variable for each displayed grid are set here
    ;Finaly adds the row and column header texts and the data to be displayed
    ;on the visible cells
    ;}  
    GridsArray(ThisGrid)\FirstVisibleColumn = 1
    GridsArray(ThisGrid)\FirstVisibleRow = 1
    ReDim GridsArray(ThisGrid)\RowHeights(Rows)
    ReDim GridsArray(ThisGrid)\ColWidths(Columns)
    ;Clear any data in grid pixel maps
    ClearMap(GridsArray(ThisGrid)\RowPixels())
    ClearMap(GridsArray(ThisGrid)\ColPixels())
    ;Default Row Heights
    For iLoop = 0 To Rows
      GridsArray(ThisGrid)\RowHeights(iLoop) = 20
    Next iLoop
    ;default Column Widths
    For iLoop = 0 To Columns
      GridsArray(ThisGrid)\ColWidths(iLoop) = 50 ;#defaultColumnWidth
    Next iLoop   

  EndProcedure

  Procedure GetColPix(ThisGrid.i,ColNum.i)
    ;{ ==Get Column Pixel Values Comment==============================
    ;Procedure to set the CellLeft and CellRight pixel values for a given column
    ;CellLeft and CellRight are Global to the module and used to disply text in a cell
    ;}
  
    CellLeft = GridsArray(ThisGrid)\Width
    CellRight = 0
  
    ForEach GridsArray(ThisGrid)\ColPixels()

      If GridsArray(ThisGrid)\ColPixels() = ColNum
        If Val(MapKey(GridsArray(ThisGrid)\ColPixels())) < Cellleft 
          CellLeft = Val(MapKey(GridsArray(ThisGrid)\ColPixels()))
        EndIf
        If Val(MapKey(GridsArray(ThisGrid)\ColPixels())) > CellRight 
          CellRight = Val(MapKey(GridsArray(ThisGrid)\ColPixels()))
        EndIf      
      EndIf
    Next
    If CellRight > GridsArray(ThisGrid)\Width
      CellRight = GridsArray(ThisGrid)\Width
    EndIf

  EndProcedure

  Procedure GetRowPix(ThisGrid.i,RowNum.i)
    ;{ ==Get Row Pixel Values Comment==============================
    ;Procedure to set the CellTop and CellBottom pixel values for a given row
    ;CellTop and CellBottom are Global to the module and used to disply text in a cell
    ;}
  
    CellTop = GridsArray(ThisGrid)\Height
    CellBottom = 0
  
    ForEach GridsArray(ThisGrid)\RowPixels()

      If GridsArray(ThisGrid)\RowPixels() = RowNum
        If Val(MapKey(GridsArray(ThisGrid)\RowPixels())) < CellTop 
          CellTop = Val(MapKey(GridsArray(ThisGrid)\RowPixels()))
        EndIf
        If Val(MapKey(GridsArray(ThisGrid)\RowPixels())) > CellBottom 
          CellBottom = Val(MapKey(GridsArray(ThisGrid)\RowPixels()))
        EndIf      
      EndIf
    Next
    If CellBottom > GridsArray(ThisGrid)\Height
      CellBottom = GridsArray(ThisGrid)\Height
    EndIf
  
  EndProcedure

  Procedure SetcellText(Grid.i,Row,Column,Text.s)
    ;{ ==SetcellText Comment==============================
    ;Procedure to draw text into a given cell on the active grid
    ;Used to set header texts as well as data into cells
    ;}
  
    ;Set the pixel values for the given cell
    GetColPix(Grid,Column) 
    GetRowPix(Grid,Row) 

    ;Add 2 pixels to Left and Top to move text awsay from the cell border
    Define Startx = CellLeft + 2 
    Define Starty = CellTop + 2 
  
    ;Draw The Text
    StartDrawing(CanvasOutput(GridsArray(Grid)\GridGadget))
      DrawingMode(#PB_2DDrawing_Default|#PB_2DDrawing_Transparent) 
      DrawText(Startx,Starty, Text,RGB(0,0,0))
    StopDrawing()

  EndProcedure

  Procedure AutoSizeColumns(Grid.i,Array GridData.s(2))
    
    Define ColumnWidth = 0
    StartDrawing(CanvasOutput(GridsArray(Grid)\GridGadget))
      For ColumnLoop = 0 To ArraySize(Griddata() , 2) 
        ColumnWidth = 0
        For j = 0 To ArraySize(Griddata() , 1) 
          If TextWidth(Griddata(j,ColumnLoop)) > ColumnWidth
            ColumnWidth = TextWidth(Griddata(j,ColumnLoop)) 
          EndIf
        Next j
        If ColumnWidth + 10 > #defaultColumnWidth
          GridsArray(Grid)\ColWidths(ColumnLoop) = ColumnWidth + 10
  
        Else
          
          GridsArray(Grid)\ColWidths(ColumnLoop) = #defaultColumnWidth

        EndIf
    
      Next ColumnLoop
    StopDrawing()

  EndProcedure

  Procedure DrawGrid(Gadget.i,Array GridData.s(2))
    ;{ ==DrawGrid Comment==============================
    ;Procedure to draw the active grid
    ;The procedure draws all grid lines and header highlights
    ;The LastVisibleRow and LastVisibleColumn variable for each displayed grid are set here
    ;Finaly adds the row and column header texts and the data to be displayed
    ;on the visible cells
    ;}
    
    Define Starty.i,Startx.i ,ThisGrid.i
    
    ;Which Grid to draw?
    ThisGrid = -1
    For iLoop = 0 To   ArraySize(GridsArray()) - 1
      
      If Gadget = GridsArray(iLoop)\GridGadget
        ThisGrid = iLoop
      EndIf
      
    Next iLoop
    If ThisGrid = -1 ;Grid Not Defined So Fail
      ProcedureReturn -1
    EndIf
    
    ;Clear Grid Canvas gadget
    Define ClearImg.i
    ClearImg = CreateImage(#PB_Any,700,400,32,RGB(255,255,255))
    SetGadgetAttribute(GridsArray(ThisGrid)\GridGadget,#PB_Canvas_Image,ImageID(ClearImg))    
    FreeImage(ClearImg)
    
    ;User wants the columns autosized
    If GridsArray(ThisGrid)\AutoCol = #True
      AutoSizeColumns(ThisGrid, GridData())
    EndIf
    
    ;Displayed portion of grid maybe smaller than canvas so
    GetGridDimensions(ThisGrid)
     
    ;Draw row and column headers
    StartDrawing(CanvasOutput(Gadget))
      DrawingMode(#PB_2DDrawing_Default|#PB_2DDrawing_Transparent)
    
      ;Column Header Row
      ;Box(0,0,GridsArray(ThisGrid)\Width,GridsArray(ThisGrid)\RowHeights(0) + 1,RGB(153,204,255))
      Box(0,0,DisplayGridWidth,GridsArray(ThisGrid)\RowHeights(0) + 1,RGB(153,204,255))      
      
      
      ;Add this Row To Map
      For i = 0 To GridsArray(ThisGrid)\RowHeights(0)
        GridsArray(ThisGrid)\RowPixels(Str(i)) = 0 
      Next
      starty = GridsArray(ThisGrid)\RowHeights(0) 
    
      ;Row Header Column
      Box(0,0,GridsArray(ThisGrid)\ColWidths(0) + 1,DisplayGridHeight,RGB(153,204,255))      
      
      ;Add this Column To Map
      For i = 0 To GridsArray(ThisGrid)\ColWidths(0)
        GridsArray(ThisGrid)\ColPixels(Str(i)) = 0 
      Next
      Startx = GridsArray(ThisGrid)\ColWidths(0)
    
      ;All Other Rows
      DrawingMode(#PB_2DDrawing_Outlined|#PB_2DDrawing_Transparent)  
      For iLoop = GridsArray(ThisGrid)\FirstVisibleRow To ArraySize(GridsArray(ThisGrid)\RowHeights())

        ;Box(0,Starty,GridsArray(ThisGrid)\Width,GridsArray(ThisGrid)\RowHeights(iLoop) + 1,RGB(0,0,0))
        Box(0,Starty,DisplayGridWidth,GridsArray(ThisGrid)\RowHeights(iLoop) + 1,RGB(0,0,0))      
        
        ;Add This Row To Map
        For i = Starty + 1 To Starty + GridsArray(ThisGrid)\RowHeights(iLoop)
          GridsArray(ThisGrid)\RowPixels(Str(i)) = iLoop
        Next i
      
        Starty = Starty + GridsArray(ThisGrid)\RowHeights(iLoop)   
        If Starty => GridsArray(ThisGrid)\Height Or iLoop = ArraySize(GridsArray(ThisGrid)\RowHeights())
          ;No More Rows Visible
          GridsArray(ThisGrid)\LastVisibleRow = iLoop
          Break
        EndIf
      Next iLoop

      ;All other columns
      For iLoop = GridsArray(ThisGrid)\FirstVisibleColumn To ArraySize(GridsArray(ThisGrid)\ColWidths())
        Box(Startx,0,GridsArray(ThisGrid)\ColWidths(iLoop) + 1,DisplayGridHeight,RGB(0,0,0))   
        ;Add This column To Map
        For i = Startx + 1 To Startx + GridsArray(ThisGrid)\ColWidths(iLoop)
          GridsArray(ThisGrid)\ColPixels(Str(i)) = iLoop
        Next i   
        Startx = Startx + GridsArray(ThisGrid)\ColWidths(iLoop)
    
        If Startx => GridsArray(ThisGrid)\Width Or iLoop = ArraySize(GridsArray(ThisGrid)\ColWidths())
          ;No More Columns Visible
          GridsArray(ThisGrid)\LastVisibleColumn = iLoop
          Break
        EndIf
      Next iLoop
     
    StopDrawing() 
    
    ;Show the Grid content
   
    ;Row Header
    For RowLoop = GridsArray(ThisGrid)\FirstVisibleRow To GridsArray(ThisGrid)\LastVisibleRow
      SetcellText(ThisGrid,RowLoop,0,Griddata(RowLoop,0)) 
    Next RowLoop
    
    ;Column Header
    For ColumnLoop = GridsArray(ThisGrid)\FirstVisibleColumn To GridsArray(ThisGrid)\LastVisibleColumn 
      SetcellText(ThisGrid,0,ColumnLoop,Griddata(0,ColumnLoop)) 
    Next ColumnLoop     

    ;Data
    For RowLoop = GridsArray(ThisGrid)\FirstVisibleRow To GridsArray(ThisGrid)\LastVisibleRow
      For ColumnLoop = GridsArray(ThisGrid)\FirstVisibleColumn To GridsArray(ThisGrid)\LastVisibleColumn 
        SetcellText(ThisGrid,RowLoop,ColumnLoop,Griddata(RowLoop,ColumnLoop)) 
      Next ColumnLoop     
    Next RowLoop
  
    ;Grid display dimensions maybe smaller than canvas gadget
    ;GetGridDimensions()


  EndProcedure
    
  Procedure New(Left.i,Top.i,Width.i,Height.i,Rows.i,Columns.i)
    
    Define iLoop.i,NewGrid.i,ThisGrid.i
    
    ThisGrid = ArraySize(GridsArray()) ;Use last element to store new grid definition
    ;GridWidth = Width
    ;GridsArray(ThisGrid)\Height = Height
    NewGrid = CanvasGadget(#PB_Any, Left,Top,Width,Height)
    GridsArray(ThisGrid)\GridGadget = NewGrid ;Set gadget number into grid array
    GridsArray(ThisGrid)\Width = Width
    GridsArray(ThisGrid)\Height = Height
    ReDim GridsArray(ArraySize(GridsArray()) + 1) ;Get ready for next grid
    
    ;Set default values for the new grid
    SetDefaults(ThisGrid,Rows,Columns)
  
    ProcedureReturn NewGrid
    
  EndProcedure

EndModule
; IDE Options = PureBasic 5.50 (Windows - x64)
; CursorPosition = 371
; FirstLine = 95
; Folding = DAAa-
; EnableXP
; EnableUnicode